#ifndef SDLSYSTEM_SYSTEM_H
#define SDLSYSTEM_SYSTEM_H

#include <vector>
#include <string>
#include "math/vec.h"
#include "system/system.h"

#include <SDL/SDL.h>

class SDLSystem : public PSystem {
public:
	SDLSystem() {reset(); }
	~SDLSystem() {shutdown(); }

	virtual void init();
	virtual void shutdown();
	
	virtual void loop(void (*loopCallback)());
	virtual void requestExit();
	bool wasExitRequested() { return exitRequested; }

	virtual void setInitDoubleBuffered(bool _initDoubleBuffered) { initDoubleBuffered = _initDoubleBuffered; }
	virtual void setInitPos(const vec2i &_initPos) { initPos = _initPos; }
	virtual void setInitSize(const vec2i &_initSize) { initSize = _initSize; }

	virtual const std::string &getArg(int index) const { return args[index]; }
	virtual size_t getArgCount() const { return args.size(); }

	//this needs to be public 'cuz of the GLUT using methods as callbacks, etc
	void doLoop();

	//for all those who want to swap/update display buffers manually inter-render
	void updateDisplayBuffers();

//created by init
	std::vector<std::string> args;

protected:
	void reset();

	//internal loop methods
	//for now I'm going to use a callback method... maybe switch it to an (abstract?) object later...
	void (*loopCallback)();
	void preUpdate();
	void postUpdate();
	
	bool initDoubleBuffered;
	bool initFullScreen;
	vec2i initPos;
	vec2i initSize;
	int initBPP;

	bool exitRequested;
	bool doubleBuffered;

	SDL_Surface *screen;
};

#endif	//SDLSYSTEM_SYSTEM_H
